# mypy: disable-error-code="index,union-attr"
import logging

import msgspec
from google.oauth2._service_account_async import Credentials

from lueur.make_id import make_id
from lueur.models import GCPMeta, Resource
from lueur.platform.gcp.client import AuthorizedSession, Client

__all__ = ["explore_memorystore"]
logger = logging.getLogger("lueur.lib")


async def explore_memorystore(
    project: str, creds: Credentials | None = None
) -> list[Resource]:
    resources = []

    async with Client("https://redis.googleapis.com", creds) as c:
        buckets = await explore_all_instances(c, project)
        resources.extend(buckets)

    return resources


###############################################################################
# Private functions
###############################################################################
async def explore_all_instances(
    c: AuthorizedSession, project: str
) -> list[Resource]:
    response = await c.get(f"/v1/projects/{project}/locations/-/instances")

    stores = msgspec.json.decode(response.content)

    if response.status_code == 403:
        logger.warning(f"Memorystore API access failure: {stores}")
        return []

    if "instances" not in stores:
        logger.warning(f"No memorystore instances found: {stores}")
        return []

    results = []
    for store in stores["instances"]:
        results.append(
            Resource(
                id=make_id(store["name"]),
                meta=GCPMeta(
                    name=store["name"],
                    display=store["displayName"],
                    kind="instance",
                    platform="gcp",
                    project=project,
                    zone=store.get("locationId"),
                    category="memorystore",
                ),
                struct=store,
            )
        )

    return results



import sys
from pathlib import Path
from typing import Any

import pytest

sys.path.append(Path.cwd())


@pytest.fixture(scope="session")
def gcp_org_id() -> str:
    return "myorg"


@pytest.fixture(scope="session")
def gcp_project_number() -> str:
    return "608773013422"


@pytest.fixture(scope="session")
def gcp_project() -> str:
    return "dummy"


@pytest.fixture(scope="session")
def gcp_region() -> str:
    return "us-east1"


@pytest.fixture(scope="session")
def gcp_cloudrun_service_A() -> str:
    return "serviceA"


@pytest.fixture(scope="session")
def gcp_cloudrun_service_B() -> str:
    return "serviceB"


@pytest.fixture(scope="session")
def gcp_cloudrun_service_account_A(
    gcp_project_number: str, gcp_cloudrun_service_A: str
) -> str:
    return (
        f"{gcp_cloudrun_service_A}-yxvjljk5@{gcp_project_number}.iam.gserviceaccount.com",
    )


@pytest.fixture(scope="session")
def gcp_cloudrun_service_B() -> str:
    return "serviceB"


@pytest.fixture(scope="session")
def gcp_cloudrun_service_account_B(
    gcp_project_number: str, gcp_cloudrun_service_B: str
) -> str:
    return (
        f"{gcp_cloudrun_service_B}-pxwytzo5@{gcp_project_number}.iam.gserviceaccount.com",
    )


@pytest.fixture(scope="session")
def gcp_gke_cluster_name() -> str:
    return "serviceB"


@pytest.fixture(scope="session")
def gcp_sql_instance_name() -> str:
    return "demo"


@pytest.fixture(scope="session")
def gcp_sql_instance_user_name() -> str:
    return "demo"


@pytest.fixture(scope="session")
def gcp_sql_instance_database_name() -> str:
    return "demo"


@pytest.fixture(scope="session")
def gcp_cloudrun_services(
    gcp_project: str,
    gcp_region: str,
    gcp_cloudrun_service_A: str,
    gcp_cloudrun_service_account_A: str,
) -> list[dict[str, Any]]:
    return [
        {
            "name": f"projects/{gcp_project}/locations/{gcp_region}/services/{gcp_cloudrun_service_account_A}",
            "uid": "78617702-cd59-4f3a-b2bb-128c9a899664",
            "generation": "1314",
            "createTime": "2023-02-10T21:23:44.062037Z",
            "updateTime": "2024-06-23T21:06:57.505454Z",
            "creator": f"{gcp_cloudrun_service_A}@chaosiq-devops.iam.gserviceaccount.com",
            "lastModifier": f"{gcp_cloudrun_service_A}@chaosiq-devops.iam.gserviceaccount.com",
            "client": "terraform",
            "ingress": "INGRESS_TRAFFIC_INTERNAL_LOAD_BALANCER",
            "launchStage": "GA",
            "template": {
                "scaling": {"maxInstanceCount": 30},
                "vpcAccess": {
                    "connector": f"projects/{gcp_project}/locations/{gcp_region}/connectors/api",
                    "egress": "PRIVATE_RANGES_ONLY",
                },
                "timeout": "60s",
                "serviceAccount": gcp_cloudrun_service_account_A,
                "containers": [
                    {
                        "image": f"gcp_cloudrun_service_account_A:9f19ab2cb8a3b4a62300ca35efcdb6486b2fd90b",
                        "resources": {
                            "limits": {"cpu": "1.0", "memory": "256Mi"},
                            "cpuIdle": True,
                            "startupCpuBoost": True,
                        },
                        "ports": [{"name": "http1", "containerPort": 8080}],
                        "startupProbe": {
                            "timeoutSeconds": 240,
                            "periodSeconds": 240,
                            "failureThreshold": 1,
                            "tcpSocket": {"port": 8080},
                        },
                    }
                ],
                "maxInstanceRequestConcurrency": 80,
            },
            "traffic": [
                {
                    "type": "TRAFFIC_TARGET_ALLOCATION_TYPE_LATEST",
                    "percent": 100,
                }
            ],
            "observedGeneration": "1314",
            "terminalCondition": {
                "type": "Ready",
                "state": "CONDITION_SUCCEEDED",
                "lastTransitionTime": "2024-06-23T21:07:06.717538Z",
            },
            "conditions": [
                {
                    "type": "RoutesReady",
                    "state": "CONDITION_SUCCEEDED",
                    "lastTransitionTime": "2024-06-23T21:07:06.684537Z",
                },
                {
                    "type": "ConfigurationsReady",
                    "state": "CONDITION_SUCCEEDED",
                    "lastTransitionTime": "2023-05-29T12:39:00.276862Z",
                },
            ],
            "latestReadyRevision": f"projects/{gcp_project}/locations/{gcp_region}/services/{gcp_cloudrun_service_A}/revisions/{gcp_cloudrun_service_A}-01314-6nm",
            "latestCreatedRevision": f"projects/{gcp_project}/locations/{gcp_region}/services/{gcp_cloudrun_service_A}/revisions/{gcp_cloudrun_service_A}-01314-6nm",
            "trafficStatuses": [
                {
                    "type": "TRAFFIC_TARGET_ALLOCATION_TYPE_LATEST",
                    "percent": 100,
                }
            ],
            "uri": f"https://{gcp_cloudrun_service_A}-h2oyppsxyz-ew.a.run.app",
            "etag": '"CPGc4rMGELC7gvEB/cHJvamVjdHMvc3RhZ2luZzEwMDIyMDIzL2xvY2F0aW9ucy9ldXJvcGUtd2VzdDEvc2VydmljZXMvcmVsaWFibHktZnJvbnRlbmQ"',
        }
    ]


@pytest.fixture(scope="session")
def gcp_cloudrun_connectors(
    gcp_project: str,
    gcp_region: str,
) -> list[dict[str, Any]]:
    return [
        {
            "name": f"projects/{gcp_project}/locations/{gcp_region}/connectors/api",
            "network": f"{gcp_project}-vpc",
            "ipCidrRange": "10.8.0.0/28",
            "state": "READY",
            "minThroughput": 200,
            "maxThroughput": 300,
            "connectedProjects": ["{gcp_project}"],
            "machineType": "e2-micro",
            "minInstances": 2,
            "maxInstances": 3,
            "createTime": "2023-02-10T17:56:37.239876Z",
            "lastRestartTime": "2024-06-25T13:55:19.639192Z",
        }
    ]


@pytest.fixture(scope="session")
def gcp_gke_clusters(
    gcp_project: str,
    gcp_region: str,
    gcp_gke_cluster_name: str,
) -> list[dict[str, Any]]:
    return [
        {
            "name": gcp_gke_cluster_name,
            "nodeConfig": {
                "machineType": "e2-medium",
                "diskSizeGb": 100,
                "oauthScopes": [
                    "https://www.googleapis.com/auth/cloud-platform"
                ],
                "metadata": {"disable-legacy-endpoints": "true"},
                "imageType": "COS_CONTAINERD",
                "labels": {"env": "demo"},
                "serviceAccount": f"gke-demo@{gcp_project}.iam.gserviceaccount.com",
                "diskType": "pd-balanced",
                "workloadMetadataConfig": {"mode": "GKE_METADATA"},
                "shieldedInstanceConfig": {"enableIntegrityMonitoring": True},
                "resourceLabels": {"env": "demo"},
                "windowsNodeConfig": {},
            },
            "masterAuth": {
                "clientCertificateConfig": {},
                "clusterCaCertificate": "...",
            },
            "loggingService": "logging.googleapis.com/kubernetes",
            "monitoringService": "monitoring.googleapis.com/kubernetes",
            "network": "k8s-dummy",
            "clusterIpv4Cidr": "10.21.0.0/14",
            "addonsConfig": {
                "httpLoadBalancing": {},
                "horizontalPodAutoscaling": {},
                "kubernetesDashboard": {"disabled": True},
                "networkPolicyConfig": {"disabled": True},
                "dnsCacheConfig": {},
                "gcePersistentDiskCsiDriverConfig": {"enabled": True},
                "gcpFilestoreCsiDriverConfig": {"enabled": True},
                "gcsFuseCsiDriverConfig": {},
            },
            "subnetwork": "k8s-dummy",
            "nodePools": [
                {
                    "name": "pool-2",
                    "config": {
                        "machineType": "e2-medium",
                        "diskSizeGb": 100,
                        "oauthScopes": [
                            "https://www.googleapis.com/auth/cloud-platform"
                        ],
                        "metadata": {"disable-legacy-endpoints": "true"},
                        "imageType": "COS_CONTAINERD",
                        "labels": {"env": "demo"},
                        "serviceAccount": f"gke-demo@{gcp_project}.iam.gserviceaccount.com",
                        "diskType": "pd-balanced",
                        "workloadMetadataConfig": {"mode": "GKE_METADATA"},
                        "shieldedInstanceConfig": {
                            "enableIntegrityMonitoring": True
                        },
                        "resourceLabels": {"env": "demo"},
                        "windowsNodeConfig": {},
                    },
                    "initialNodeCount": 1,
                    "autoscaling": {
                        "enabled": True,
                        "maxNodeCount": 1,
                        "locationPolicy": "BALANCED",
                    },
                    "management": {"autoUpgrade": True, "autoRepair": True},
                    "maxPodsConstraint": {"maxPodsPerNode": "110"},
                    "podIpv4CidrSize": 24,
                    "locations": [
                        f"{gcp_region}-c",
                        f"{gcp_region}-d",
                        f"{gcp_region}-b",
                    ],
                    "networkConfig": {
                        "podRange": "gke-{gcp_gke_cluster_name}-pods-8ba2b403",
                        "podIpv4CidrBlock": "10.21.0.0/14",
                        "enablePrivateNodes": False,
                        "podIpv4RangeUtilization": 0.003,
                    },
                    "selfLink": f"https://container.googleapis.com/v1/projects/{gcp_project}/locations/{gcp_region}/clusters/{gcp_gke_cluster_name}/nodePools/pool-2",
                    "version": "1.28.9-gke.1209000",
                    "instanceGroupUrls": [
                        f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/zones/{gcp_region}-c/instanceGroupManagers/gke-{gcp_gke_cluster_name}-pool-2-ff7287e7-grp",
                        f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/zones/{gcp_region}-d/instanceGroupManagers/gke-{gcp_gke_cluster_name}-pool-2-691ec5a9-grp",
                        f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/zones/{gcp_region}-b/instanceGroupManagers/gke-{gcp_gke_cluster_name}-pool-2-06a83df0-grp",
                    ],
                    "status": "RUNNING",
                    "upgradeSettings": {"maxSurge": 1, "strategy": "SURGE"},
                    "etag": "85bfa51c-efe1-4cc5-b723-7843f72e3dbb",
                },
                {
                    "name": "pool-3",
                    "config": {
                        "machineType": "n2-standard-2",
                        "diskSizeGb": 100,
                        "oauthScopes": [
                            "https://www.googleapis.com/auth/cloud-platform"
                        ],
                        "metadata": {"disable-legacy-endpoints": "true"},
                        "imageType": "COS_CONTAINERD",
                        "labels": {"env": "demo"},
                        "serviceAccount": f"demo-cluster@{gcp_project}.iam.gserviceaccount.com",
                        "diskType": "pd-balanced",
                        "workloadMetadataConfig": {"mode": "GKE_METADATA"},
                        "shieldedInstanceConfig": {
                            "enableIntegrityMonitoring": True
                        },
                        "advancedMachineFeatures": {
                            "enableNestedVirtualization": False
                        },
                        "resourceLabels": {"env": "demo"},
                        "windowsNodeConfig": {},
                    },
                    "initialNodeCount": 2,
                    "autoscaling": {
                        "enabled": True,
                        "maxNodeCount": 3,
                        "locationPolicy": "BALANCED",
                    },
                    "management": {"autoUpgrade": True, "autoRepair": True},
                    "maxPodsConstraint": {"maxPodsPerNode": "110"},
                    "podIpv4CidrSize": 24,
                    "locations": [
                        f"{gcp_region}-c",
                        f"{gcp_region}-d",
                        f"{gcp_region}-b",
                    ],
                    "networkConfig": {
                        "podRange": "gke-{gcp_gke_cluster_name}-pods-8ba2b403",
                        "podIpv4CidrBlock": "10.21.0.0/14",
                        "enablePrivateNodes": False,
                        "podIpv4RangeUtilization": 0.003,
                    },
                    "selfLink": f"https://container.googleapis.com/v1/projects/{gcp_project}/locations/{gcp_region}/clusters/{gcp_gke_cluster_name}/nodePools/pool-3",
                    "version": "1.28.9-gke.1209000",
                    "instanceGroupUrls": [
                        f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/zones/{gcp_region}-c/instanceGroupManagers/gke-{gcp_gke_cluster_name}-pool-3-ea9a9827-grp",
                        f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/zones/{gcp_region}-d/instanceGroupManagers/gke-{gcp_gke_cluster_name}-pool-3-a5ac3c22-grp",
                        f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/zones/{gcp_region}-b/instanceGroupManagers/gke-{gcp_gke_cluster_name}-pool-3-e1aa9d04-grp",
                    ],
                    "status": "RUNNING",
                    "upgradeSettings": {"maxSurge": 1, "strategy": "SURGE"},
                    "etag": "4fae315e-2b4c-4173-a724-126d2dfe99d0",
                    "queuedProvisioning": {},
                },
            ],
            "locations": [
                f"{gcp_region}-c",
                f"{gcp_region}-d",
                f"{gcp_region}-b",
            ],
            "resourceLabels": {"env": "demo"},
            "labelFingerprint": "d26e484d",
            "legacyAbac": {},
            "ipAllocationPolicy": {
                "useIpAliases": True,
                "clusterIpv4Cidr": "10.21.0.0/14",
                "servicesIpv4Cidr": "10.232.208.0/20",
                "clusterSecondaryRangeName": f"gke-{gcp_gke_cluster_name}-pods-8ba2b403",
                "servicesSecondaryRangeName": f"gke-{gcp_gke_cluster_name}-services-8ba2b403",
                "clusterIpv4CidrBlock": "10.21.0.0/14",
                "servicesIpv4CidrBlock": "10.232.208.0/20",
                "stackType": "IPV4",
                "podCidrOverprovisionConfig": {},
                "defaultPodIpv4RangeUtilization": 0.003,
            },
            "masterAuthorizedNetworksConfig": {
                "gcpPublicCidrsAccessEnabled": True
            },
            "maintenancePolicy": {
                "window": {
                    "recurringWindow": {
                        "window": {
                            "startTime": "2024-03-11T23:00:00Z",
                            "endTime": "2024-03-12T23:00:00Z",
                        },
                        "recurrence": "FREQ=WEEKLY;BYDAY=MO,TU,WE,TH,FR,SA,SU",
                    }
                },
                "resourceVersion": "1c4b3335",
            },
            "binaryAuthorization": {"evaluationMode": "DISABLED"},
            "autoscaling": {
                "enableNodeAutoprovisioning": True,
                "resourceLimits": [
                    {"resourceType": "cpu", "minimum": "1", "maximum": "1"},
                    {"resourceType": "memory", "minimum": "1", "maximum": "1"},
                ],
                "autoscalingProfile": "BALANCED",
                "autoprovisioningNodePoolDefaults": {
                    "oauthScopes": [
                        "https://www.googleapis.com/auth/userinfo.email",
                        "https://www.googleapis.com/auth/cloud-platform",
                    ],
                    "serviceAccount": "default",
                    "upgradeSettings": {"maxSurge": 1, "strategy": "SURGE"},
                    "management": {"autoUpgrade": True, "autoRepair": True},
                    "diskSizeGb": 100,
                    "diskType": "pd-balanced",
                    "shieldedInstanceConfig": {
                        "enableIntegrityMonitoring": True
                    },
                    "imageType": "COS_CONTAINERD",
                },
                "autoprovisioningLocations": [
                    f"{gcp_region}-c",
                    f"{gcp_region}-d",
                    f"{gcp_region}-b",
                ],
            },
            "networkConfig": {
                "network": f"projects/{gcp_project}/global/networks/k8s-dummy",
                "subnetwork": f"projects/{gcp_project}/regions/{gcp_region}/subnetworks/k8s-dummy",
                "defaultSnatStatus": {},
                "datapathProvider": "ADVANCED_DATAPATH",
                "serviceExternalIpsConfig": {},
                "gatewayApiConfig": {"channel": "CHANNEL_STANDARD"},
            },
            "defaultMaxPodsConstraint": {"maxPodsPerNode": "110"},
            "authenticatorGroupsConfig": {},
            "privateClusterConfig": {
                "privateEndpoint": "10.0.10.20",
                "publicEndpoint": "104.155.86.88",
            },
            "databaseEncryption": {"state": "DECRYPTED"},
            "shieldedNodes": {"enabled": True},
            "releaseChannel": {"channel": "REGULAR"},
            "workloadIdentityConfig": {
                "workloadPool": f"{gcp_project}.svc.id.goog"
            },
            "costManagementConfig": {"enabled": True},
            "notificationConfig": {"pubsub": {}},
            "selfLink": f"https://container.googleapis.com/v1/projects/{gcp_project}/locations/{gcp_region}/clusters/{gcp_gke_cluster_name}",
            "zone": "{gcp_region}",
            "endpoint": "104.155.86.88",
            "initialClusterVersion": "1.27.8-gke.1067004",
            "currentMasterVersion": "1.28.9-gke.1209000",
            "currentNodeVersion": "1.28.9-gke.1209000",
            "createTime": "2024-03-13T16:51:17+00:00",
            "status": "RUNNING",
            "servicesIpv4Cidr": "10.232.208.0/20",
            "instanceGroupUrls": [
                f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/zones/{gcp_region}-c/instanceGroupManagers/gke-{gcp_gke_cluster_name}-pool-2-ff7287e7-grp",
                f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/zones/{gcp_region}-d/instanceGroupManagers/gke-{gcp_gke_cluster_name}-pool-2-691ec5a9-grp",
                f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/zones/{gcp_region}-b/instanceGroupManagers/gke-{gcp_gke_cluster_name}-pool-2-06a83df0-grp",
                f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/zones/{gcp_region}-c/instanceGroupManagers/gke-{gcp_gke_cluster_name}-pool-3-ea9a9827-grp",
                f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/zones/{gcp_region}-d/instanceGroupManagers/gke-{gcp_gke_cluster_name}-pool-3-a5ac3c22-grp",
                f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/zones/{gcp_region}-b/instanceGroupManagers/gke-{gcp_gke_cluster_name}-pool-3-e1aa9d04-grp",
            ],
            "currentNodeCount": 3,
            "location": gcp_region,
            "autopilot": {},
            "id": "8ba2b4038c744fa5bd026220abc6db5978a99ba6eb574f82958cedfb89bee63a",
            "nodePoolDefaults": {
                "nodeConfigDefaults": {
                    "loggingConfig": {"variantConfig": {"variant": "DEFAULT"}}
                }
            },
            "loggingConfig": {
                "componentConfig": {
                    "enableComponents": ["SYSTEM_COMPONENTS", "WORKLOADS"]
                }
            },
            "monitoringConfig": {
                "componentConfig": {"enableComponents": ["SYSTEM_COMPONENTS"]},
                "managedPrometheusConfig": {"enabled": True},
                "advancedDatapathObservabilityConfig": {},
            },
            "etag": "aa55d4ac-c142-4eb7-8819-f5a71eb1b34f",
            "securityPostureConfig": {
                "mode": "BASIC",
                "vulnerabilityMode": "VULNERABILITY_DISABLED",
            },
            "enterpriseConfig": {"clusterTier": "STANDARD"},
        }
    ]


@pytest.fixture(scope="session")
def gcp_gke_cluster_nodepools(
    gcp_project: str,
    gcp_region: str,
    gcp_gke_cluster_name: str,
) -> list[dict[str, Any]]:
    return [
        {
            "name": "pool-2",
            "config": {
                "machineType": "e2-medium",
                "diskSizeGb": 100,
                "oauthScopes": [
                    "https://www.googleapis.com/auth/cloud-platform"
                ],
                "metadata": {"disable-legacy-endpoints": "true"},
                "imageType": "COS_CONTAINERD",
                "labels": {"env": "demo"},
                "serviceAccount": f"gke-demo@{gcp_project}.iam.gserviceaccount.com",
                "diskType": "pd-balanced",
                "workloadMetadataConfig": {"mode": "GKE_METADATA"},
                "shieldedInstanceConfig": {"enableIntegrityMonitoring": True},
                "resourceLabels": {"env": "demo"},
                "windowsNodeConfig": {},
            },
            "initialNodeCount": 1,
            "autoscaling": {
                "enabled": True,
                "maxNodeCount": 1,
                "locationPolicy": "BALANCED",
            },
            "management": {"autoUpgrade": True, "autoRepair": True},
            "maxPodsConstraint": {"maxPodsPerNode": "110"},
            "podIpv4CidrSize": 24,
            "locations": [
                f"{gcp_region}-c",
                f"{gcp_region}-d",
                f"{gcp_region}-b",
            ],
            "networkConfig": {
                "podRange": "gke-{gcp_gke_cluster_name}-pods-8ba2b403",
                "podIpv4CidrBlock": "10.21.0.0/14",
                "enablePrivateNodes": False,
            },
            "selfLink": f"https://container.googleapis.com/v1/projects/{gcp_project}/locations/{gcp_region}/clusters/{gcp_gke_cluster_name}/nodePools/pool-2",
            "version": "1.28.9-gke.1209000",
            "instanceGroupUrls": [
                f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/zones/{gcp_region}-c/instanceGroupManagers/gke-{gcp_gke_cluster_name}-pool-2-ff7287e7-grp",
                f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/zones/{gcp_region}-d/instanceGroupManagers/gke-{gcp_gke_cluster_name}-pool-2-691ec5a9-grp",
                f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/zones/{gcp_region}-b/instanceGroupManagers/gke-{gcp_gke_cluster_name}-pool-2-06a83df0-grp",
            ],
            "status": "RUNNING",
            "upgradeSettings": {"maxSurge": 1, "strategy": "SURGE"},
            "etag": "85bfa51c-efe1-4cc5-b723-7843f72e3dbb",
        },
        {
            "name": "pool-3",
            "config": {
                "machineType": "n2-standard-2",
                "diskSizeGb": 100,
                "oauthScopes": [
                    "https://www.googleapis.com/auth/cloud-platform"
                ],
                "metadata": {"disable-legacy-endpoints": "true"},
                "imageType": "COS_CONTAINERD",
                "labels": {"env": "demo"},
                "serviceAccount": f"demo-cluster@{gcp_project}.iam.gserviceaccount.com",
                "diskType": "pd-balanced",
                "workloadMetadataConfig": {"mode": "GKE_METADATA"},
                "shieldedInstanceConfig": {"enableIntegrityMonitoring": True},
                "advancedMachineFeatures": {
                    "enableNestedVirtualization": False
                },
                "resourceLabels": {"env": "demo"},
                "windowsNodeConfig": {},
            },
            "initialNodeCount": 2,
            "autoscaling": {
                "enabled": True,
                "maxNodeCount": 3,
                "locationPolicy": "BALANCED",
            },
            "management": {"autoUpgrade": True, "autoRepair": True},
            "maxPodsConstraint": {"maxPodsPerNode": "110"},
            "podIpv4CidrSize": 24,
            "locations": [
                f"{gcp_region}-c",
                f"{gcp_region}-d",
                f"{gcp_region}-b",
            ],
            "networkConfig": {
                "podRange": f"gke-{gcp_gke_cluster_name}-pods-8ba2b403",
                "podIpv4CidrBlock": "10.21.0.0/14",
                "enablePrivateNodes": False,
            },
            "selfLink": f"https://container.googleapis.com/v1/projects/{gcp_project}/locations/{gcp_region}/clusters/{gcp_gke_cluster_name}/nodePools/pool-3",
            "version": "1.28.9-gke.1209000",
            "instanceGroupUrls": [
                f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/zones/{gcp_region}-c/instanceGroupManagers/gke-{gcp_gke_cluster_name}-pool-3-ea9a9827-grp",
                f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/zones/{gcp_region}-d/instanceGroupManagers/gke-{gcp_gke_cluster_name}-pool-3-a5ac3c22-grp",
                f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/zones/{gcp_region}-b/instanceGroupManagers/gke-{gcp_gke_cluster_name}-pool-3-e1aa9d04-grp",
            ],
            "status": "RUNNING",
            "upgradeSettings": {"maxSurge": 1, "strategy": "SURGE"},
            "etag": "4fae315e-2b4c-4173-a724-126d2dfe99d0",
            "queuedProvisioning": {},
        },
    ]


@pytest.fixture(scope="session")
def gcp_global_securities(
    gcp_project: str,
    gcp_region: str,
    gcp_gke_cluster_name: str,
) -> list[dict[str, Any]]:
    return [
        {
            "kind": "compute#securityPolicy",
            "id": "2953638056173439792",
            "creationTimestamp": "2023-02-10T13:23:43.497-08:00",
            "name": "cloud-armor-waf-policy",
            "rules": [
                {
                    "kind": "compute#securityPolicyRule",
                    "description": "",
                    "priority": 1000,
                    "match": {
                        "expr": {
                            "expression": "evaluatePreconfiguredWaf('sqli-v33-stable', {'sensitivity': 1}) || evaluatePreconfiguredWaf('sqli-v33-stable', {'sensitivity': 2})"
                        }
                    },
                    "action": "deny(403)",
                    "preview": False,
                    "headerAction": {},
                },
                {
                    "kind": "compute#securityPolicyRule",
                    "description": "",
                    "priority": 1001,
                    "match": {
                        "expr": {
                            "expression": "evaluatePreconfiguredWaf('xss-v33-stable', {'sensitivity': 1})"
                        }
                    },
                    "action": "deny(403)",
                    "preview": False,
                    "headerAction": {},
                },
                {
                    "kind": "compute#securityPolicyRule",
                    "description": "",
                    "priority": 1002,
                    "match": {
                        "expr": {
                            "expression": "evaluatePreconfiguredExpr('lfi-v33-stable')"
                        }
                    },
                    "action": "deny(403)",
                    "preview": False,
                    "headerAction": {},
                },
                {
                    "kind": "compute#securityPolicyRule",
                    "description": "",
                    "priority": 1003,
                    "match": {
                        "expr": {
                            "expression": "evaluatePreconfiguredWaf('rce-v33-stable', {'sensitivity': 1}) || evaluatePreconfiguredWaf('rce-v33-stable', {'sensitivity': 3})"
                        }
                    },
                    "action": "deny(403)",
                    "preview": False,
                    "headerAction": {},
                },
                {
                    "kind": "compute#securityPolicyRule",
                    "description": "",
                    "priority": 1004,
                    "match": {
                        "expr": {
                            "expression": "evaluatePreconfiguredExpr('rfi-v33-stable')"
                        }
                    },
                    "action": "deny(403)",
                    "preview": False,
                    "headerAction": {},
                },
                {
                    "kind": "compute#securityPolicyRule",
                    "description": "",
                    "priority": 1005,
                    "match": {
                        "expr": {
                            "expression": "evaluatePreconfiguredExpr('scannerdetection-v33-stable')"
                        }
                    },
                    "action": "deny(403)",
                    "preview": False,
                    "headerAction": {},
                },
                {
                    "kind": "compute#securityPolicyRule",
                    "description": "",
                    "priority": 1006,
                    "match": {
                        "expr": {
                            "expression": "evaluatePreconfiguredExpr('protocolattack-v33-stable')"
                        }
                    },
                    "action": "deny(403)",
                    "preview": False,
                    "headerAction": {},
                },
                {
                    "kind": "compute#securityPolicyRule",
                    "description": "",
                    "priority": 1008,
                    "match": {
                        "expr": {
                            "expression": "evaluatePreconfiguredExpr('cve-canary')"
                        }
                    },
                    "action": "deny(403)",
                    "preview": False,
                    "headerAction": {},
                },
                {
                    "kind": "compute#securityPolicyRule",
                    "description": "Default allow all rule",
                    "priority": 2147483647,
                    "match": {
                        "versionedExpr": "SRC_IPS_V1",
                        "config": {"srcIpRanges": ["*"]},
                    },
                    "action": "allow",
                    "preview": False,
                    "headerAction": {},
                },
            ],
            "fingerprint": "C80RqDu3TxA=",
            "selfLink": "https://www.googleapis.com/compute/v1/projects/{gcp_project}/global/securityPolicies/cloud-armor-waf-policy",
            "type": "CLOUD_ARMOR",
            "labelFingerprint": "43WpSiB8rYM=",
        }
    ]


@pytest.fixture(scope="session")
def gcp_vpc_networks(
    gcp_project: str,
    gcp_region: str,
) -> list[dict[str, Any]]:
    return [
        {
            "kind": "compute#network",
            "id": "8198762671780854345",
            "creationTimestamp": "2023-08-18T03:25:27.855-07:00",
            "name": f"{gcp_project}-demo-vpc",
            "selfLink": f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/global/networks/{gcp_project}-demo-vpc",
            "selfLinkWithId": f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/global/networks/8198762671780854345",
            "autoCreateSubnetworks": False,
            "subnetworks": [
                f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/regions/{gcp_region}/subnetworks/{gcp_project}-demo-subnet"
            ],
            "peerings": [
                {
                    "name": "servicenetworking-googleapis-com",
                    "network": "https://www.googleapis.com/compute/v1/projects/e36e7da775b77ab11p-tp/global/networks/servicenetworking",
                    "state": "ACTIVE",
                    "stateDetails": "[2023-08-22T03:19:16.069-07:00]: Connected.",
                    "autoCreateRoutes": True,
                    "exportCustomRoutes": False,
                    "importCustomRoutes": False,
                    "exchangeSubnetRoutes": True,
                    "exportSubnetRoutesWithPublicIp": False,
                    "importSubnetRoutesWithPublicIp": False,
                    "stackType": "IPV4_ONLY",
                }
            ],
            "routingConfig": {"routingMode": "GLOBAL"},
            "networkFirewallPolicyEnforcementOrder": "AFTER_CLASSIC_FIREWALL",
        },
        {
            "kind": "compute#network",
            "id": "276817910687905639",
            "creationTimestamp": "2023-02-10T09:56:14.453-08:00",
            "name": f"{gcp_project}-vpc",
            "selfLink": f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/global/networks/{gcp_project}-vpc",
            "selfLinkWithId": f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/global/networks/276817910687905639",
            "autoCreateSubnetworks": False,
            "subnetworks": [
                f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/regions/{gcp_region}/subnetworks/{gcp_project}-subnet",
            ],
            "peerings": [
                {
                    "name": "servicenetworking-googleapis-com",
                    "network": "https://www.googleapis.com/compute/v1/projects/xa321698e3390dfc6p-tp/global/networks/servicenetworking",
                    "state": "ACTIVE",
                    "stateDetails": "[2023-02-10T09:59:15.029-08:00]: Connected.",
                    "autoCreateRoutes": True,
                    "exportCustomRoutes": True,
                    "importCustomRoutes": False,
                    "exchangeSubnetRoutes": True,
                    "exportSubnetRoutesWithPublicIp": False,
                    "importSubnetRoutesWithPublicIp": False,
                    "stackType": "IPV4_ONLY",
                }
            ],
            "routingConfig": {"routingMode": "GLOBAL"},
            "networkFirewallPolicyEnforcementOrder": "AFTER_CLASSIC_FIREWALL",
        },
    ]


@pytest.fixture(scope="session")
def gcp_vpc_subnets(
    gcp_project: str,
    gcp_region: str,
    gcp_gke_cluster_name: str,
) -> list[dict[str, Any]]:
    return [
        {
            "kind": "compute#subnetwork",
            "id": "9081669609185438966",
            "creationTimestamp": "2024-03-07T07:17:05.764-08:00",
            "name": "k8s-dummy",
            "description": "",
            "network": f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/global/networks/k8s-dummy",
            "ipCidrRange": "10.0.10.0/24",
            "gatewayAddress": "10.0.10.1",
            "region": f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/regions/{gcp_region}",
            "selfLink": f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/regions/{gcp_region}/subnetworks/k8s-dummy",
            "privateIpGoogleAccess": True,
            "secondaryIpRanges": [
                {
                    "rangeName": f"gke-{gcp_gke_cluster_name}-pods-8ba2b403",
                    "ipCidrRange": "10.236.0.0/14",
                    "reservedInternalRange": f"https://networkconnectivity.googleapis.com/v1/projects/{gcp_project}/locations/global/internalRanges/gke-{gcp_gke_cluster_name}-pods-8ba2b403",
                },
                {
                    "rangeName": f"gke-{gcp_gke_cluster_name}-services-8ba2b403",
                    "ipCidrRange": "10.232.208.0/20",
                    "reservedInternalRange": f"https://networkconnectivity.googleapis.com/v1/projects/{gcp_project}/locations/global/internalRanges/gke-{gcp_gke_cluster_name}-services-8ba2b403",
                },
            ],
            "fingerprint": "dzixcuBWUFA=",
            "enableFlowLogs": False,
            "privateIpv6GoogleAccess": "DISABLE_GOOGLE_ACCESS",
            "purpose": "PRIVATE",
            "logConfig": {
                "enable": False,
                "aggregationInterval": "INTERVAL_5_SEC",
                "flowSampling": 0.5,
                "metadata": "INCLUDE_ALL_METADATA",
            },
            "stackType": "IPV4_ONLY",
        },
        {
            "kind": "compute#subnetwork",
            "id": "2343700906476575489",
            "creationTimestamp": "2023-08-18T03:25:50.276-07:00",
            "name": f"{gcp_project}-demo-subnet",
            "network": f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/global/networks/{gcp_project}-demo-vpc",
            "ipCidrRange": "10.10.11.0/24",
            "gatewayAddress": "10.10.11.1",
            "region": f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/regions/{gcp_region}",
            "selfLink": f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/regions/{gcp_region}/subnetworks/{gcp_project}-demo-subnet",
            "privateIpGoogleAccess": False,
            "fingerprint": "J9WCTYERFvs=",
            "enableFlowLogs": False,
            "privateIpv6GoogleAccess": "DISABLE_GOOGLE_ACCESS",
            "purpose": "PRIVATE",
            "logConfig": {
                "enable": False,
                "aggregationInterval": "INTERVAL_5_SEC",
                "flowSampling": 0.5,
                "metadata": "INCLUDE_ALL_METADATA",
            },
            "stackType": "IPV4_ONLY",
        },
        {
            "kind": "compute#subnetwork",
            "id": "47568586173111221787",
            "creationTimestamp": "2023-02-10T09:56:38.882-08:00",
            "name": f"{gcp_project}-subnet",
            "network": f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/global/networks/{gcp_project}-vpc",
            "ipCidrRange": "10.10.10.0/24",
            "gatewayAddress": "10.10.10.1",
            "region": f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/regions{gcp_region}",
            "selfLink": f"https://www.googleapis.com/compute/v1/projects/{gcp_project}/regions/{gcp_region}/subnetworks/{gcp_project}-subnet",
            "privateIpGoogleAccess": False,
            "fingerprint": "cAOmFIA_hYt=",
            "enableFlowLogs": False,
            "privateIpv6GoogleAccess": "DISABLE_GOOGLE_ACCESS",
            "purpose": "PRIVATE",
            "logConfig": {
                "enable": False,
                "aggregationInterval": "INTERVAL_5_SEC",
                "flowSampling": 0.5,
                "metadata": "INCLUDE_ALL_METADATA",
            },
            "stackType": "IPV4_ONLY",
        },
    ]


@pytest.fixture(scope="session")
def gcp_sql_instances(
    gcp_project: str,
    gcp_project_number: str,
    gcp_region: str,
    gcp_sql_instance_name: str,
) -> list[dict[str, Any]]:
    return [
        {
            "kind": "sql#instance",
            "state": "RUNNABLE",
            "databaseVersion": "POSTGRES_14",
            "settings": {
                "authorizedGaeApplications": [],
                "tier": "db-g1-small",
                "kind": "sql#settings",
                "availabilityType": "REGIONAL",
                "pricingPlan": "PER_USE",
                "replicationType": "SYNCHRONOUS",
                "activationPolicy": "ALWAYS",
                "ipConfiguration": {
                    "privateNetwork": f"projects/{gcp_project}/global/networks/{gcp_project}-demo-vpc",
                    "authorizedNetworks": [],
                    "sslMode": "TRUSTED_CLIENT_CERTIFICATE_REQUIRED",
                    "ipv4Enabled": False,
                    "requireSsl": True,
                    "enablePrivatePathForGoogleCloudServices": False,
                },
                "locationPreference": {
                    "zone": "{gcp_region}-b",
                    "kind": "sql#locationPreference",
                },
                "dataDiskType": "PD_SSD",
                "maintenanceWindow": {
                    "kind": "sql#maintenanceWindow",
                    "hour": 0,
                    "day": 0,
                },
                "backupConfiguration": {
                    "startTime": "09:00",
                    "kind": "sql#backupConfiguration",
                    "location": "eu",
                    "backupRetentionSettings": {
                        "retentionUnit": "COUNT",
                        "retainedBackups": 7,
                    },
                    "enabled": True,
                    "replicationLogArchivingEnabled": True,
                    "pointInTimeRecoveryEnabled": True,
                    "transactionLogRetentionDays": 7,
                    "transactionalLogStorageState": "CLOUD_STORAGE",
                },
                "insightsConfig": {},
                "connectorEnforcement": "NOT_REQUIRED",
                "settingsVersion": "92",
                "storageAutoResizeLimit": "0",
                "storageAutoResize": True,
                "dataDiskSizeGb": "10",
                "deletionProtectionEnabled": False,
            },
            "etag": "6b1b361752ce3be3d99c9a56bcfac9d0d7819769cc12eb3b66ad4e7550bb5bd6",
            "failoverReplica": {"available": True},
            "ipAddresses": [{"type": "PRIVATE", "ipAddress": "10.53.0.2"}],
            "serverCaCert": {
                "kind": "sql#sslCert",
                "certSerialNumber": "0",
                "cert": "...",
                "commonName": "...",
                "sha1Fingerprint": "4e1243bd22c66e76c2ba9eddc1f91394e57f9f83",
                "instance": gcp_sql_instance_name,
                "createTime": "2023-08-18T10:29:22.350Z",
                "expirationTime": "2033-08-15T10:30:22.350Z",
            },
            "instanceType": "CLOUD_SQL_INSTANCE",
            "project": gcp_project,
            "serviceAccountEmailAddress": f"p{gcp_project_number}-3iu1fg@gcp-sa-cloud-sql.iam.gserviceaccount.com",
            "backendType": "SECOND_GEN",
            "selfLink": f"https://sqladmin.googleapis.com/v1/projects/{gcp_project}/instances/{gcp_sql_instance_name}",
            "connectionName": f"{gcp_project}:{gcp_region}:{gcp_sql_instance_name}",
            "name": gcp_sql_instance_name,
            "region": "{gcp_region}",
            "gceZone": "{gcp_region}-b",
            "secondaryGceZone": "{gcp_region}-d",
            "databaseInstalledVersion": "POSTGRES_14_12",
            "maintenanceVersion": "POSTGRES_14_12.R20240514.00_04",
            "createTime": "2023-08-18T10:26:43.081Z",
            "sqlNetworkArchitecture": "NEW_NETWORK_ARCHITECTURE",
        }
    ]


@pytest.fixture(scope="session")
def gcp_sql_users(
    gcp_project: str,
    gcp_sql_instance_user_name: str,
    gcp_sql_instance_name: str,
) -> list[dict[str, Any]]:
    return [
        {
            "kind": "sql#user",
            "etag": "...",
            "name": gcp_sql_instance_user_name,
            "host": "",
            "instance": gcp_sql_instance_name,
            "project": gcp_project,
        }
    ]


@pytest.fixture(scope="session")
def gcp_sql_databases(
    gcp_project: str,
    gcp_sql_instance_name: str,
    gcp_sql_instance_database_name: str,
) -> list[dict[str, Any]]:
    return [
        {
            "kind": "sql#database",
            "charset": "UTF8",
            "collation": "en_US.UTF8",
            "etag": "...",
            "name": "postgres",
            "instance": gcp_sql_instance_name,
            "selfLink": f"https://sqladmin.googleapis.com/v1/projects/{gcp_project}/instances/{gcp_sql_instance_name}/databases/{gcp_sql_instance_database_name}",
            "project": gcp_project,
        }
    ]


@pytest.fixture(scope="session")
def gcp_monitoring_services(
    gcp_project: str,
    gcp_region: str,
    gcp_gke_cluster_name: str,
    gcp_project_number: str,
) -> list[dict[str, Any]]:
    return [
        {
            "name": f"projects/{gcp_project_number}/services/demo-service",
            "displayName": "Demo service",
            "cloudRun": {"serviceName": "demo", "location": gcp_region},
            "basicService": {
                "serviceType": "CLOUD_RUN",
                "serviceLabels": {
                    "location": gcp_region,
                    "service_name": "demo",
                },
            },
        },
        {
            "name": f"projects/{gcp_project_number}/services/ist:{gcp_project}-location-{gcp_region}-{gcp_gke_cluster_name}-default-demo",
            "displayName": "consumer",
            "telemetry": {
                "resourceName": f"//container.googleapis.com/projects/{gcp_project}/locations/{gcp_region}/clusters/{gcp_gke_cluster_name}/k8s/namespaces/default/services/demo"
            },
            "gkeService": {
                "projectId": gcp_project,
                "location": gcp_region,
                "clusterName": gcp_gke_cluster_name,
                "namespaceName": "default",
                "serviceName": "demo",
            },
            "basicService": {
                "serviceType": "GKE_SERVICE",
                "serviceLabels": {
                    "cluster_name": gcp_gke_cluster_name,
                    "location": gcp_region,
                    "namespace_name": "default",
                    "project_id": gcp_project,
                    "service_name": "demo",
                },
            },
        },
    ]


@pytest.fixture(scope="session")
def gcp_monitoring_slo(
    gcp_project: str,
    gcp_region: str,
    gcp_project_number: str,
    gcp_gke_cluster_name: str,
) -> list[dict[str, Any]]:
    return [
        {
            "name": f"projects/{gcp_project_number}/services/demo-service/serviceLevelObjectives/ePaA7SMgBHqIeTyyiQyrZg",
            "serviceLevelIndicator": {
                "requestBased": {
                    "goodTotalRatio": {
                        "goodServiceFilter": f'resource.labels.project_id = "{gcp_project}" AND resource.type = "cloud_run_revision" AND resource.labels.location = "{gcp_region}" AND resource.labels.service_name = "demo" AND metric.type = "run.googleapis.com/request_count" AND metric.labels.response_code_class = "2xx"',
                        "badServiceFilter": f'resource.labels.project_id = "{gcp_project}" AND resource.type = "cloud_run_revision" AND resource.labels.location = "{gcp_region}" AND resource.labels.service_name = "demo" AND metric.type = "run.googleapis.com/request_count" AND metric.labels.response_code_class = "5xx"',
                    }
                }
            },
            "goal": 0.95,
            "calendarPeriod": "WEEK",
            "displayName": "95% - Availability - Calendar week",
        },
        {
            "name": f"projects/{gcp_project_number}/services/demo-service/serviceLevelObjectives/ly7qEQ1yRUaRMnGhT8Us8b",
            "serviceLevelIndicator": {
                "requestBased": {
                    "distributionCut": {
                        "distributionFilter": f'resource.labels.project_id = "{gcp_project}" AND resource.type = "cloud_run_revision" AND resource.labels.location = "{gcp_region}" AND resource.labels.service_name = "demo" AND metric.type = "run.googleapis.com/request_latencies" AND metric.labels.response_code_class = "2xx"',
                        "range": {"min": "-Infinity", "max": 250},
                    }
                }
            },
            "goal": 0.95,
            "calendarPeriod": "WEEK",
            "displayName": "95% - Latency - Calendar week",
        },
        {
            "name": f"projects/{gcp_project_number}/services/ist:{gcp_project}-location-{gcp_region}-{gcp_gke_cluster_name}-default-demo/serviceLevelObjectives/PO1ph13MYbCQaTgXIwUYLg",
            "serviceLevelIndicator": {
                "windowsBased": {
                    "goodTotalRatioThreshold": {
                        "performance": {
                            "distributionCut": {
                                "distributionFilter": 'metric.type="loadbalancing.googleapis.com/https/total_latencies" resource.type="https_lb_rule" metric.labels.response_code="200" resource.labels.backend_name="k8s1-f2804333-default-demo-8080-7ype3f1a" resource.labels.backend_target_name="gkegw1-wwbg-default-demo-8080-k9o8ddmf9emo"',
                                "range": {"min": -9007199254740991, "max": 400},
                            }
                        },
                        "threshold": 0.95,
                    },
                    "windowPeriod": "60s",
                }
            },
            "goal": 0.98,
            "calendarPeriod": "DAY",
            "displayName": "98% - Consumer E2E Latency - Calendar day",
        },
    ]


@pytest.fixture(scope="session")
def gcp_monitoring_alert_policies(
    gcp_project: str,
    gcp_region: str,
    gcp_project_number: str,
    gcp_gke_cluster_name: str,
) -> list[dict[str, Any]]:
    return [
        {
            "name": f"projects/{gcp_project}/alertPolicies/15766192705010179244",
            "displayName": "Burn rate on 98% - Consumer E2E Latency - Calendar day",
            "combiner": "OR",
            "creationRecord": {
                "mutateTime": "2024-05-30T19:32:01.159000572Z",
                "mutatedBy": "me@example.com",
            },
            "mutationRecord": {
                "mutateTime": "2024-05-30T20:06:57.817712837Z",
                "mutatedBy": "me@example.com",
            },
            "conditions": [
                {
                    "conditionThreshold": {
                        "filter": f'select_slo_burn_rate("projects/{gcp_project_number}/services/ist:{gcp_project}-location-{gcp_region}-{gcp_gke_cluster_name}-default-demo/serviceLevelObjectives/PO1ph13MYbCQaTgXIwUYLg", "60s")',
                        "comparison": "COMPARISON_GT",
                        "thresholdValue": 10,
                        "duration": "0s",
                        "trigger": {"count": 1},
                        "aggregations": [
                            {
                                "alignmentPeriod": "300s",
                                "perSeriesAligner": "ALIGN_MAX",
                            }
                        ],
                    },
                    "displayName": "Burn rate on 98% - Consumer E2E Latency - Calendar day",
                    "name": f"projects/{gcp_project}/alertPolicies/15766192705010179244/conditions/4203981762528173346",
                }
            ],
            "notificationChannels": [
                f"projects/{gcp_project}/notificationChannels/15347963261329368790"
            ],
            "enabled": True,
        }
    ]


@pytest.fixture(scope="session")
def gcp_monitoring_notification_channels(
    gcp_project: str,
    gcp_region: str,
    gcp_project_number: str,
    gcp_gke_cluster_name: str,
) -> list[dict[str, Any]]:
    return [
        {
            "type": "slack",
            "displayName": "Boom",
            "labels": {
                "auth_token": "....",
                "team": "Team",
                "channel_name": "#alerts",
            },
            "name": f"projects/{gcp_project}]/notificationChannels/15347963261329368790",
            "enabled": True,
            "creationRecord": {"mutateTime": "2024-05-21T12:50:59.495660677Z"},
            "mutationRecords": [
                {"mutateTime": "2024-05-21T12:50:59.495660677Z"}
            ],
        }
    ]



# mypy: disable-error-code="index,union-attr"
from concurrent.futures import ThreadPoolExecutor
from typing import Any

from lueur.links import add_link
from lueur.make_id import make_id
from lueur.models import AWSMeta, Discovery, Link, Resource
from lueur.platform.aws.client import Client
from lueur.rules import iter_resource

__all__ = ["explore_vpc", "expand_links"]


def explore_vpc(region: str) -> list[Resource]:
    resources = []

    vpcs = explore_vpcs(region)
    resources.extend(vpcs)

    if not vpcs:
        return resources

    futures = []
    with ThreadPoolExecutor(max_workers=len(vpcs)) as executor:
        for vpc in vpcs:
            futures.append(
                executor.submit(explore_subnets, region, vpc.meta.name)
            )

    for future in futures:
        resources.extend(future.result())

    return resources


###############################################################################
# Private functions
###############################################################################
def explore_vpcs(region: str) -> list[Resource]:
    results = []

    with Client("ec2", region) as c:
        vpcs = c.describe_vpcs()

        for vpc in vpcs["Vpcs"]:
            results.append(
                Resource(
                    id=make_id(vpc["VpcId"]),
                    meta=AWSMeta(
                        name=vpc["VpcId"],
                        display=vpc["VpcId"],
                        kind="vpc",
                        platform="aws",
                        region=region,
                        category="network",
                    ),
                    struct=vpc,
                )
            )

    return results


def explore_subnets(region: str, vpc_id: str) -> list[Resource]:
    with Client("ec2", region) as c:
        subnets = c.describe_subnets(
            Filters=[
                {
                    "Name": "vpc-id",
                    "Values": [
                        vpc_id,
                    ],
                },
            ],
        )

    results = []
    for subnet in subnets["Subnets"]:
        results.append(
            Resource(
                id=make_id(subnet["SubnetId"]),
                meta=AWSMeta(
                    name=subnet["SubnetId"],
                    display=subnet["SubnetId"],
                    kind="subnet",
                    platform="aws",
                    region=region,
                    category="network",
                ),
                struct=subnet,
            )
        )

    return results


def expand_links(d: Discovery, serialized: dict[str, Any]) -> None:
    for vpc_name in iter_resource(
        serialized,
        "$.resources[?@.meta.kind=='vpc'].meta.name",
    ):
        r_id = vpc_name.parent.parent.obj["id"]
        vpc_id = vpc_name.value
        p = f"$.resources[?@.meta.kind=='subnet' && @.struct.VpcId=='{vpc_id}']"
        for listener in iter_resource(serialized, p):
            add_link(
                d,
                r_id,
                Link(
                    direction="out",
                    kind="subnet",
                    path=listener.path,
                    pointer=str(listener.pointer()),
                ),
            )



# mypy: disable-error-code="index,union-attr"
import asyncio
import logging

import msgspec
from google.oauth2._service_account_async import Credentials

from lueur.make_id import make_id
from lueur.models import GCPMeta, Resource
from lueur.platform.gcp.client import AuthorizedSession, Client
from lueur.platform.gcp.zone import list_project_zones

__all__ = ["explore_compute"]
logger = logging.getLogger("lueur.lib")


async def explore_compute(
    project: str, creds: Credentials | None = None
) -> list[Resource]:
    zones = await list_project_zones(project, creds)

    resources: list[Resource] = []

    if not zones:
        return resources

    async with Client("https://compute.googleapis.com", creds) as c:
        buckets = await explore_all_zones_instances(c, project, zones)
        resources.extend(buckets)

    return resources


###############################################################################
# Private functions
###############################################################################
async def explore_all_zones_instances(
    c: AuthorizedSession,
    project: str,
    zones: list[str],
) -> list[Resource]:
    resources = []
    tasks: list[asyncio.Task] = []

    async with asyncio.TaskGroup() as tg:
        for zone in zones:
            tasks.append(
                tg.create_task(explore_zone_instances(c, project, zone))
            )

    for task in tasks:
        result = task.result()
        if result is None:
            continue
        resources.extend(result)

    return resources


async def explore_zone_instances(
    c: AuthorizedSession,
    project: str,
    zone: str,
) -> list[Resource]:
    response = await c.get(
        f"/compute/v1/projects/{project}/zones/{zone}/instances"
    )

    instances = msgspec.json.decode(response.content)

    if response.status_code == 403:
        logger.warning(f"Compute API access failure: {instances}")
        return []

    if "warning" in instances:
        w = instances["warning"]
        logger.warning(f"Error when exploiring GCP instances: {w}")
        return []

    if "items" not in instances:
        logger.warning(f"No compute instances found: {instances}")
        return []

    results = []

    for instance in instances["items"]:
        self_link = instance.get("selfLink")

        results.append(
            Resource(
                id=make_id(instance["id"]),
                meta=GCPMeta(
                    name=instance["name"],
                    display=instance["name"],
                    kind="instance",
                    platform="gcp",
                    project=project,
                    zone=instance.get("zone"),
                    self_link=self_link,
                    category="compute",
                ),
                struct=instance,
            )
        )

    return results



# mypy: disable-error-code="index,union-attr"
import logging

import msgspec
from google.oauth2._service_account_async import Credentials

from lueur.make_id import make_id
from lueur.models import GCPMeta, Resource
from lueur.platform.gcp.client import AuthorizedSession, Client

__all__ = ["explore_storage"]
logger = logging.getLogger("lueur.lib")


async def explore_storage(
    project: str, creds: Credentials | None = None
) -> list[Resource]:
    resources = []

    async with Client("https://storage.googleapis.com", creds) as c:
        buckets = await explore_buckets(c, project)
        resources.extend(buckets)

    return resources


###############################################################################
# Private functions
###############################################################################
async def explore_buckets(c: AuthorizedSession, project: str) -> list[Resource]:
    response = await c.get("/storage/v1/b", params={"project": project})

    buckets = msgspec.json.decode(response.content)

    if response.status_code == 403:
        logger.warning(f"Storage API access failure: {buckets}")
        return []

    if "items" not in buckets:
        logger.warning(f"No buckets found: {buckets}")
        return []

    results = []
    for bucket in buckets["items"]:
        self_link = bucket.get("selfLink")

        results.append(
            Resource(
                id=make_id(bucket["id"]),
                meta=GCPMeta(
                    name=bucket["name"],
                    display=bucket["name"],
                    kind="bucket",
                    platform="gcp",
                    project=project,
                    zone=bucket.get("location"),
                    self_link=self_link,
                    category="storage",
                ),
                struct=bucket,
            )
        )

    return results



from typing import Any, Iterable

import jsonpath

__all__ = [
    "iter_resource",
    "match_path",
]


def iter_resource(
    discovery: dict[str, Any], path: str
) -> Iterable[jsonpath.JSONPathMatch]:
    yield from jsonpath.finditer(path, discovery)


def match_path(
    discovery: dict[str, Any], path: str
) -> jsonpath.JSONPathMatch | None:
    return jsonpath.match(path, discovery)



from typing import Any

import msgspec
import pytest
import respx

from lueur.platform.gcp.cloudrun import explore_cloudrun, expand_links

from tests.platform.gcp.utils import patch_auth


@respx.mock
@pytest.mark.asyncio
async def test_explore_cloudrun(
    gcp_project: str,
    gcp_region: str,
    gcp_cloudrun_services: list[dict[str, Any]],
    gcp_cloudrun_connectors: list[dict[str, Any]],
) -> None:
    respx.post(
        "https://oauth2.googleapis.com/token",
    ).respond(content=msgspec.json.encode({"access_token": "xyz"}))

    respx.get(
        f"https://run.googleapis.com/v2/projects/{gcp_project}/locations/{gcp_region}/services",
    ).respond(content=msgspec.json.encode({"services": gcp_cloudrun_services}))

    respx.get(
        f"https://vpcaccess.googleapis.com/v1beta1/projects/{gcp_project}/locations/{gcp_region}/connectors",
    ).respond(
        content=msgspec.json.encode({"connectors": gcp_cloudrun_connectors})
    )

    with patch_auth():
        resources = await explore_cloudrun(gcp_project, gcp_region)

    assert len(resources) == 2



# mypy: disable-error-code="index,union-attr"
import asyncio
import logging
from typing import Any

import msgspec
from google.oauth2._service_account_async import Credentials

from lueur.links import add_link
from lueur.make_id import make_id
from lueur.models import Discovery, GCPMeta, Link, Resource
from lueur.platform.gcp.client import AuthorizedSession, Client
from lueur.rules import iter_resource

__all__ = ["explore_vpc", "expand_links"]
logger = logging.getLogger("lueur.lib")


async def explore_vpc(
    project: str, creds: Credentials | None = None
) -> list[Resource]:
    resources = []

    async with Client("https://compute.googleapis.com", creds) as c:
        networks = await explore_global_networks(c, project)
        resources.extend(networks)

        tasks: list[asyncio.Task] = []
        seen: list[str] = []

        async with asyncio.TaskGroup() as tg:
            for net in networks:
                for subnet in net.struct["subnetworks"]:
                    if subnet not in seen:
                        seen.append(subnet)
                        tasks.append(
                            tg.create_task(explore_subnet(c, project, subnet))
                        )

        for task in tasks:
            result = task.result()
            if result is None:
                continue
            resources.append(result)

    return resources


###############################################################################
# Private functions
###############################################################################
async def explore_global_networks(
    c: AuthorizedSession, project: str
) -> list[Resource]:
    response = await c.get(f"/compute/v1/projects/{project}/global/networks")

    networks = msgspec.json.decode(response.content)

    if response.status_code == 403:
        logger.warning(f"Network API access failure: {networks}")
        return []

    if "items" not in networks:
        logger.warning(f"No global networks found: {networks}")
        return []

    results = []
    for net in networks["items"]:
        self_link = net["selfLink"]

        results.append(
            Resource(
                id=make_id(net["id"]),
                meta=GCPMeta(
                    name=net["name"],
                    display=net["name"],
                    kind="network",
                    project=project,
                    self_link=self_link,
                    platform="gcp",
                    category="network",
                ),
                struct=net,
            )
        )

    return results


async def explore_subnet(
    c: AuthorizedSession, project: str, subnet: str
) -> Resource | None:
    response = await c.get(subnet)

    sub = msgspec.json.decode(response.content)

    if "error" in sub:
        return None

    self_link = sub["selfLink"]

    return Resource(
        id=make_id(sub["id"]),
        meta=GCPMeta(
            name=sub["id"],
            display=sub["name"],
            kind="subnet",
            project=project,
            self_link=self_link,
            platform="gcp",
            category="network",
        ),
        struct=sub,
    )


def expand_links(d: Discovery, serialized: dict[str, Any]) -> None:
    for s in iter_resource(
        serialized, "$.resources[?@.meta.kind=='network'].meta.self_link"
    ):
        r_id = s.parent.parent.obj["id"]  # type: ignore
        net = s.value
        p = f"$.resources[?@.meta.kind=='subnet' && @.struct.network=='{net}']"
        for subnet in iter_resource(serialized, p):
            add_link(
                d,
                r_id,
                Link(
                    direction="out",
                    kind="subnet",
                    path=subnet.path,
                    pointer=str(subnet.pointer()),
                ),
            )

        p = f"$.resources[?@.meta.kind=='global-neg' && @.struct.network=='{net}']"  # noqa E501
        for neg in iter_resource(serialized, p):
            add_link(
                d,
                r_id,
                Link(
                    direction="out",
                    kind="global-neg",
                    path=neg.path,
                    pointer=str(neg.pointer()),
                ),
            )

        p = f"$.resources[?@.meta.kind=='regional-neg' && @.struct.network=='{net}']"  # noqa E501
        for neg in iter_resource(serialized, p):
            add_link(
                d,
                r_id,
                Link(
                    direction="out",
                    kind="regional-neg",
                    path=neg.path,
                    pointer=str(neg.pointer()),
                ),
            )

    for s in iter_resource(
        serialized,
        "$.resources[?@.meta.kind=='network' && @.meta.name=='default']",
    ):
        r_id = s.obj["id"]
        p = "$.resources[?@.meta.kind=='regional-neg' && @.struct.network!='']"
        for neg in iter_resource(serialized, p):
            add_link(
                d,
                r_id,
                Link(
                    direction="out",
                    kind="regional-neg",
                    path=neg.path,
                    pointer=str(neg.pointer()),
                ),
            )



from contextlib import contextmanager
from typing import Iterator, cast

import boto3
import botocore.client
from botocore.config import Config

__all__ = ["Client"]


@contextmanager
def Client(
    aws_service: str, region: str
) -> Iterator[botocore.client.BaseClient]:
    cfg = Config(region_name=region)
    c = cast(botocore.client.BaseClient, boto3.client(aws_service, config=cfg))

    try:
        yield c
    finally:
        c.close()



import asyncio
import secrets
from typing import Any

from lueur.make_id import make_id
from lueur.models import Discovery, Meta
from lueur.platform.k8s.deployment import (
    expand_links as deployment_expand_links,
)
from lueur.platform.k8s.deployment import explore_deployment
from lueur.platform.k8s.gateway import expand_links as gateway_expand_links
from lueur.platform.k8s.gateway import explore_gateway
from lueur.platform.k8s.ingress import explore_ingress
from lueur.platform.k8s.network_policy import explore_network_policy
from lueur.platform.k8s.node import expand_links as node_expand_links
from lueur.platform.k8s.node import explore_node
from lueur.platform.k8s.pod import explore_pod
from lueur.platform.k8s.replicaset import (
    expand_links as replicaset_expand_links,
)
from lueur.platform.k8s.replicaset import explore_replicaset
from lueur.platform.k8s.service import explore_service

__all__ = ["explore", "expand_links"]


async def explore() -> Discovery:
    resources = []
    tasks: list[asyncio.Task] = []

    async with asyncio.TaskGroup() as tg:
        tasks.append(tg.create_task(explore_node()))
        tasks.append(tg.create_task(explore_pod()))
        tasks.append(tg.create_task(explore_replicaset()))
        tasks.append(tg.create_task(explore_deployment()))
        tasks.append(tg.create_task(explore_ingress()))
        tasks.append(tg.create_task(explore_service()))
        tasks.append(tg.create_task(explore_network_policy()))
        tasks.append(tg.create_task(explore_gateway()))

    for task in tasks:
        result = task.result()
        if result is None:
            continue
        resources.extend(result)

    name = secrets.token_hex(8)

    return Discovery(
        id=make_id(name),
        resources=resources,
        meta=Meta(name=name, display="Kubernetes", kind="k8s", category=None),
    )


def expand_links(d: Discovery, serialized: dict[str, Any]) -> None:
    deployment_expand_links(d, serialized)
    gateway_expand_links(d, serialized)
    node_expand_links(d, serialized)
    replicaset_expand_links(d, serialized)
